@@ -15,9 +15,16 @@
  * along with OnionCat. If not, see <http://www.gnu.org/licenses/>.
  */
 
+/*! \file ocat.c
+ * This is the main file OnionCat. It initializes everything, runs all threads,
+ * and finally terminates OnionCat again.
+ * \date 2019/09/08
+ * \author Bernhard R. Fischer, <bf@abenteuerland.at>
+ */
 
 #include "ocat.h"
 #include "ocat_netdesc.h"
+#include "ocathosts.h"
 
 
 void usage(const char *s)
@@ -29,15 +36,18 @@ void usage(const char *s)
          "   -b                    daemonize (default = %d)\n"
          "   -B                    do not daemonize (default = %d)\n"
          "   -h                    display usage message\n"
-         "   -H                    toggle /etc/hosts lookup (default = %d)\n"
+         "   -H                    toggle hosts lookup (default = %d, see also option -g)\n"
          "   -C                    disable local controller interface\n"
          "   -d <n>                set debug level to n, default = %d\n"
          "   -e <ifup-script>      execute ifup-script after opening interface\n"
          "   -f <config_file>      read config from config_file (default = %s)\n"
+         "   -g <hosts_path>       set path to hosts file for hosts lookup, defaults to system hosts file, if not set.\n"
+         "                         This option implicitly activates -H.\n"
          "   -i                    convert onion hostname to IPv6 and exit\n"
          "   -I                    GarliCat mode, use I2P instead of Tor\n"
          "   -l [<ip>:]<port>      set ocat listen address and port, default = 127.0.0.1:%d\n"
          "   -L <log_file>         log output to <log_file> (default = stderr)\n"
+         "   -n <tunname>          set the tun device name, may contain format string (e.g. tun%%d)\n"
          "   -o <ipv6_addr>        convert IPv6 address to onion url and exit\n"
          "   -p                    use TAP device instead of TUN\n"
          "   -P [<pid_file>]       create pid file at location of <pid_file> (default = %s)\n"
@@ -51,6 +61,7 @@ void usage(const char *s)
          "   -U                    disable unidirectional mode\n"
          "   -u <user>             change UID to user, default = \"%s\"\n"
          "   -4                    enable IPv4 support (default = %d)\n"
+         "   -5 [socks5|direct]    use SOCKS5 or direct connections instead of SOCKS4A (default = %d)\n"
          , CNF(version), s,
          // option defaults start here
          OCAT_DIR, NDESC(clog_file), CNF(create_clog), 
@@ -61,7 +72,7 @@ void usage(const char *s)
 #ifndef WITHOUT_TUN
          TUN_DEV,
 #endif
-         OCAT_UNAME, CNF(ipv4_enable)
+         OCAT_UNAME, CNF(ipv4_enable), CNF(socks5)
             );
 }
 
@@ -335,11 +346,25 @@ int parse_opt(int argc, char *argv[])
    log_debug("parse_opt_early()");
    opterr = 1;
    optind = 1;
-   while ((c = getopt(argc, argv, "f:IabBCd:e:hHrRiopl:t:T:s:Uu:4L:P:")) != -1)
+   while ((c = getopt(argc, argv, "f:IabBCd:e:g:hHrRiopl:t:T:s:Uu:45:L:P:n:")) != -1)
    {
       log_debug("getopt(): c = %c, optind = %d, opterr = %d, optarg = \"%s\"", c, optind, opterr, SSTR(optarg));
       switch (c)
       {
+         // use SOCKS5 instead of SOCKS4A
+         case '5':
+            if (!strcasecmp(optarg, "socks5"))
+               CNF(socks5) = CONNTYPE_SOCKS5;
+            else if (!strcasecmp(optarg, "direct"))
+            {
+               CNF(socks5) = CONNTYPE_DIRECT;
+               CNF(hosts_lookup) = 1;
+               hosts_init("");
+            }
+            else
+               log_msg(LOG_ERR, "unknown type \"%s\", ignoring", optarg);
+            break;
+
          // those options are parsed in parse_opt_early()
          case 'f':
          case 'I':
@@ -369,6 +394,12 @@ int parse_opt(int argc, char *argv[])
             CNF(ifup) = optarg;
             break;
 
+         case 'g':
+            CNF(hosts_path) = optarg;
+            hosts_set_path(CNF(hosts_path));
+            CNF(hosts_lookup) = 1;
+            break;
+
          case 'i':
             urlconv = 1;
             break;
@@ -378,7 +409,7 @@ int parse_opt(int argc, char *argv[])
             exit(1);
 
          case 'H':
-            CNF(hosts_lookup) = !CNF(hosts_lookup);
+            CNF(hosts_lookup) = 1;
             break;
 
          case 'l':
@@ -403,6 +434,7 @@ int parse_opt(int argc, char *argv[])
 
          case 'p':
             CNF(use_tap) = 1;
+            CNF(ipconfig) = 0;
             break;
 
          case 'P':
@@ -437,6 +469,10 @@ int parse_opt(int argc, char *argv[])
          case 'T':
             tun_dev_ = optarg;
             break;
+
+         case 'n':
+            strlcpy(CNF(tunname), optarg, sizeof(CNF(tunname)));
+            break;
 #endif
 
          case 'U':
@@ -463,6 +499,7 @@ int parse_opt(int argc, char *argv[])
 
 int main(int argc, char *argv[])
 {
+   char *charset = "qwertyuiopasdfghjklzxcvbnmQWERTYUIOPASDFGHJKLZXCVBNM234567";
 #ifdef HAVE_GETPWNAM_R
    char pwdbuf[SIZE_1K];
 #endif
@@ -548,7 +585,7 @@ int main(int argc, char *argv[])
    // copy onion-URL from command line
    log_debug("argv[%d] = \"%s\"", optind, SSTR(argv[optind]));
    if (!CNF(rand_addr))
-      strncpy(CNF(onion_url), argv[optind], NDESC(name_size));
+      strlcpy(CNF(onion_url), argv[optind], sizeof(CNF(onion_url)));
    // ...or generate a random one
    else
       rand_onion(CNF(onion_url));
@@ -556,8 +593,20 @@ int main(int argc, char *argv[])
    // convert parameter to IPv6 address
    if ((s = strchr(CNF(onion_url), '.')))
          *s = '\0';
-   if (strlen(CNF(onion_url)) != 16)
-      log_msg(LOG_ERR, "parameter seems not to be valid onion hostname"), exit(1);
+   // check for valid onion name length (v2 -> 16, HSv3 -> 56, I2P -> 52)
+   if ((strlen(CNF(onion_url)) != 16) && (strlen(CNF(onion_url)) != CNF(l_hs_namelen)))
+      log_msg(LOG_ERR, "parameter seems not to be valid onion hostname: invalid length"), exit(1);
+   // check for valid base32 charset
+   if (strspn(CNF(onion_url), charset) != strlen(CNF(onion_url)))
+      log_msg(LOG_ERR, "parameter seems not to be valid onion hostname: invalid characters"), exit(1);
+   // if it is a v3 hostname
+   if (strlen(CNF(onion_url)) == CNF(l_hs_namelen))
+   {
+      // copy it to the dedicated v3 variable
+      strlcpy(CNF(onion3_url), CNF(onion_url), sizeof(CNF(onion3_url)));
+      // truncate name for IPv6 conversion to the lower 16 chars
+      strlcpy(CNF(onion_url), &CNF(onion_url[CNF(l_hs_namelen) - 16]), 17);
+   }
    if (oniontipv6(CNF(onion_url), &CNF(ocat_addr)) == -1)
       log_msg(LOG_ERR, "parameter seems not to be valid onion hostname"), exit(1);
    if (CNF(ipv4_enable))
@@ -670,12 +719,10 @@ int main(int argc, char *argv[])
    if (CNF(controller))
       run_ocat_thread("controller", ocat_controller, NULL);
 
-#ifdef CONNECT_ROOT_PEERS
-   // initiate connections to permanent root peers
-   log_debug("connecting root peers");
-   for (c = 0; c < ROOT_PEERS; c++)
-      if (!IN6_ARE_ADDR_EQUAL(&CNF(root_peer[c]), &CNF(ocat_addr)))
-         socks_queue(CNF(root_peer[c]), 1);
+#ifdef WITH_LOOPBACK_RESPONDER
+   // starting loopback responder
+   run_ocat_thread("lloopback", local_loopback_responder, NULL);
+   add_remote_loopback_route();
 #endif
 
    // install signal handler
