@@ -1,4 +1,4 @@
-/* Copyright 2008-2017 Bernhard R. Fischer, Daniel Haslinger.
+/* Copyright 2008-2019 Bernhard R. Fischer, Daniel Haslinger.
  *
  * This file is part of OnionCat.
  *
@@ -15,6 +15,13 @@
  * along with OnionCat. If not, see <http://www.gnu.org/licenses/>.
  */
 
+/*! \file ocat.h
+ * This file is the central header file of OnionCat. It includes all other
+ * headers and contains all macros, structures, typedefs,...
+ * \author Bernhard R. Fischer
+ * \date 2019/09/08
+ */
+
 #ifndef OCAT_H
 #define OCAT_H
 
@@ -25,6 +32,8 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
+#include <stddef.h>
+#include <stdint.h>
 #include <string.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -65,6 +74,15 @@
 #ifdef HAVE_NETINET_IN_SYSTM_H
 #include <netinet/in_systm.h>
 #endif
+#ifdef HAVE_NET_IF_H
+#include <net/if.h>
+#endif
+#ifdef HAVE_NETINET6_IN6_VAR_H
+#include <netinet6/in6_var.h>
+#endif
+#ifdef HAVE_NETINET6_ND6_H
+#include <netinet6/nd6.h>
+#endif
 #include <arpa/inet.h>
 #ifdef HAVE_NET_IF_H
 #include <net/if.h>
@@ -99,6 +117,9 @@
 #ifdef HAVE_LINUX_IF_TUN_H
 #include <linux/if_tun.h>
 #endif
+#ifdef HAVE_LINUX_IPV6_H
+#include <linux/ipv6.h>
+#endif
 #ifdef HAVE_NET_IF_TUN_H
 #include <net/if_tun.h>
 #endif
@@ -111,9 +132,11 @@
 #endif
 
 #ifndef ETHERTYPE_IP
+//! Ether type for IPv4.
 #define ETHERTYPE_IP 0x0800
 #endif
 #ifndef ETHERTYPE_IPV6
+//! Ether type for IPv6.
 #define ETHERTYPE_IPV6 0x86dd
 #endif
 
@@ -138,6 +161,7 @@
 //! Length of an .onion-URL (without ".onion" and '\0')
 #define ONION_URL_LEN 16
 
+//! Maximum number of peers allowed.
 #define MAXPEERS 1024
 #ifdef __OpenBSD__
 #define OCAT_UNAME "_tor"
@@ -146,12 +170,18 @@
 #else
 #define OCAT_UNAME "tor"
 #endif
+//! Uid of unprivileged user.
 #define OCAT_UNPRIV_UID 65534
+//! Name of unprivileged user (if system offers no name).
 #define OCAT_UNPRIV_UNAME "(unknown)"
-#define OCAT_URL "http://www.abenteuerland.at/onioncat/"
+//! Project URL of OnionCat.
+#define OCAT_URL "https://www.onioncat.org/"
+//! Path to OnionCat logging directory (see option -a).
 #define OCAT_DIR ".ocat"
 //#define OCAT_CONNECT_LOG "connect_log"
+//! Default path to PID file (option -P).
 #define PID_FILE "/var/run/ocat.pid"
+//! Name of the author of OnionCat.
 #define OCAT_AUTHOR "Bernhard R. Fischer"
 
 //! Maximum frame (packet) size, should be able to keep one maximum size ipv6-packet: 2^16 + 40 + 4
@@ -159,6 +189,7 @@
 
 //! Standard buffer size 1024 bytes
 #define SIZE_1K 1024
+//! Standard buffer size 256 bytes
 #define SIZE_256 256
 
 #define DEQUEUER_WAKEUP 3
@@ -167,9 +198,9 @@
 
 //! Maximum idle time for a peer, after that time the peer is closed.
 #define MAX_IDLE_TIME 180
-//! # of secs after a cleaner wakeup occurs
+//! \# of secs after a cleaner wakeup occurs
 #define CLEANER_WAKEUP 10
-//! # of secs after stats output is generated
+//! \# of secs after stats output is generated
 #define STAT_WAKEUP 600
 //! keepalive time
 #define KEEPALIVE_TIME 60
@@ -190,20 +221,43 @@
 
 //#define PEER_CONNECT 0
 #define PEER_ACTIVE 1
+#define PEER_DELETE 2
 
+//! Outgoing peer => connect().
 #define PEER_INCOMING 0
+//! Incoming peer => accept().
 #define PEER_OUTGOING 1
 
+//! Maximum length of thread names.
 #define THREAD_NAME_LEN 11
 //! thread stack size (default stack size on OpenBSD is too small)
 #define THREAD_STACK_SIZE 262144
 
+//! Connection type SOCKS4A (option -5).
+#define CONNTYPE_SOCKS4A 0
+//! Connection type SOCKS5 (option -5).
+#define CONNTYPE_SOCKS5 1
+//! Connection type direct, i.e. no SOCKS (option -5).
+#define CONNTYPE_DIRECT 2
+
+//! SOCKS state machine: new connection
 #define SOCKS_NEW 0
+//! SOCKS state machine: connect() in progress
 #define SOCKS_CONNECTING 1
+//! SOCKS state machine: SOCKS4A request sent
 #define SOCKS_4AREQ_SENT 2
+//! SOCKS state machine: this state is NOT used!
 #define SOCKS_4ARESPONSE 3
+//! SOCKS state machine: SOCKS5 greeting sent
+#define SOCKS_5GREET_SENT 4
+//! SOCKS state machine: SOCK5 request sent
+#define SOCKS_5REQ_SENT 5
+//! SOCKS state machine: successfully opened, ready for data transfer
+#define SOCKS_READY 126
+//! SOCKS state machine: request ready for deletion
 #define SOCKS_DELETE 127
 
+//! maximum number of SOCKS retries before becoming deleted
 #define SOCKS_MAX_RETRY 3
 
 #define E_RT_NOMEM -1
@@ -239,11 +293,14 @@
 #define MFD_SET(f,s,m) {FD_SET(f, s); m = f > m ? f : m;}
 
 //! copy an IPv6 address from b to a
-#define IN6_ADDR_COPY(a,b) *((struct in6_addr*)a)=*(struct in6_addr*)b
+//#define IN6_ADDR_COPY(a,b) *((struct in6_addr*)a)=*(struct in6_addr*)b
+#define IN6_ADDR_COPY(a,b) memcpy(a, b, sizeof(struct in6_addr))
 
+//! Index to OcatSetup.fhd_key for IPv4.
 #define IPV4_KEY 0
+//! Index to OcatSetup.fhd_key for IPv6.
 #define IPV6_KEY 1
-
+//! Macro to return size of anonymous sockaddr structure (only AF_INET and AF_INET6).
 #define SOCKADDR_SIZE(x) (((struct sockaddr*) x)->sa_family == AF_INET ? sizeof(struct sockaddr_in) : ((struct sockaddr*) x)->sa_family == AF_INET6 ? sizeof(struct sockaddr_in6) : 0)
 
 #define VERSION_STRING_LEN 256
@@ -269,40 +326,65 @@
 #endif
 
 
+//! General configuration data
+/*! OcatSetup is used as a global structure holding general configuration
+ * parameters for OnionCat.
+ */
 struct OcatSetup
 {
    //! frame header of local OS in network byte order
-   //! it is initialized in ocattun.c
+   /*! for IPV4 (IPV4_KEY => 0) and IPv6 (IPV6_KEY => 1), it is initialized in ocattun.c */
    uint32_t fhd_key[2];
+   //! size of the frame header, actually this is sizeof(uint32_t) which is 4
    int fhd_key_len;
-   //! TCP port of SOCKS port of local Tor proxy
-   //uint16_t tor_socks_port;
-   //! reload port of OnionCat listening for connections
-   //uint16_t ocat_listen_port;
    //! virtual port of OnionCat hidden service
    uint16_t ocat_dest_port;
    //! local port of controller interface
    uint16_t ocat_ctrl_port;
    //! file descriptors of TUN device (usually tunfd[0] == tunfd[1])
    int tunfd[2];
+   //! configure IP addresses on startup
+   int ipconfig;
+   //! debug level
    int debug_level;
    //! user name to change uid to
    char *usrname;
+   //! onion URL which corresponds to the IPv6 address
    char onion_url[SIZE_256];
+   //! long hs v3 onion name
+   char onion3_url[SIZE_256];
+   //! IPv6 address of OnionCat
    struct in6_addr ocat_addr;
    //! flag to create connection log
    int create_clog;
    //! flag to not change uid to unprivileged user
    int runasroot;
+   //! controller interface enabled/disabled (option -C)
    int controller;
+   //! directory where OnionCat puts the connect log
    char *ocat_dir;
-   //! name of tunnel charcter device
+   //! name of tunnel charcter device FIXME: seems to be unnused, uses tun_dev_ instead -> needs fix
    char *tun_dev;
+   //! Connection type (SOCKS4a, 5, or direct).
+   /*! use SOCKS5 (CONNTYPE_SOCKS5 => 1) or direct connects (CONNTYPE_DIRECT =>
+    * 2) instead of SOCKS4A (CONNTYPE_SOCKS4A => 0), option -5 */
+   int socks5;
+   //! length of long HS names
+   int l_hs_namelen;
    //! tunnel interface name
    char tunname[SIZE_256];
+   //! transport of IPv4 enabled (option -4)
    int ipv4_enable;
+   //! IPv4 address of OnionCat
    struct in_addr ocat_addr4;
-   int ocat_addr4_mask;
+   //! IPv4 netmask
+   union
+   {
+      //! netmask as int type
+      int ocat_addr4_mask;
+      //! netmask is struct in_addr type
+      struct in_addr ocat_addr4_netmask;
+   };
    char *config_file;
    int config_read;
    int config_failed;
@@ -316,11 +398,6 @@ struct OcatSetup
    FILE *logf;
    int use_syslog;
    int daemon;
-#ifdef CONNECT_ROOT_PEERS
-   //! hardcoded permanent peers
-#define ROOT_PEERS 1
-   struct in6_addr root_peer[ROOT_PEERS];
-#endif
    time_t uptime;
    char *frandn;
    //! destination socket address of Tor's SOCKS port
@@ -354,6 +431,7 @@ struct OcatSetup
       respect to security because both ends are verfied. */
    int unidirectional;
    int hosts_lookup;
+   char *hosts_path;       //!< path to hosts file, defaults to system hosts file if NULL
    const char *domain;     //!< domain name appended to network host name
    struct in6_addr oc_vdns;
 };
@@ -369,6 +447,7 @@ typedef struct PacketQueue
 } PacketQueue_t;
 #endif
 
+//! This structure holds the SOCKS4A header.
 typedef struct SocksHdr
 {
    char ver;
@@ -377,6 +456,7 @@ typedef struct SocksHdr
    struct in_addr addr;
 } __attribute__((packed)) SocksHdr_t;
 
+//! Structure to hold the SOCKS5 header.
 typedef struct Socks5Hdr
 {
    char ver;
@@ -386,6 +466,7 @@ typedef struct Socks5Hdr
    char addr;
 } __attribute__((packed)) Socks5Hdr_t;
 
+//! This structure holds all data associated with a peer (a remote OnionCat).
 typedef struct OcatPeer
 {
    struct OcatPeer *next;  //!< pointer to next peer in list
@@ -407,8 +488,10 @@ typedef struct OcatPeer
    time_t last_io;         //!< timestamp when last I/O packet measurement started
    unsigned inm;
    unsigned outm;
+   int rand;               //!< random peer number
 } OcatPeer_t;
 
+//! OcatThread is a control structure to manage each thread of OnionCat.
 typedef struct OcatThread
 {
    struct OcatThread *next;
@@ -419,8 +502,13 @@ typedef struct OcatThread
    char name[THREAD_NAME_LEN];
    void *(*entry)(void*);
    void *parm;
+   int ready;              //!< thread is ready, i.e. every initialization is done
 } OcatThread_t;
 
+//! Data structure for SOCKS connections in progress.
+/*! This structure holds all data associate with SOCKS connections which are
+ * currently in opening state, i.e. not connected but trying to connect.
+ */
 typedef struct SocksQueue
 {
    struct SocksQueue *next;
@@ -460,6 +548,7 @@ struct ip6_psh
    uint8_t nxt;
 } __attribute__((packed));
 
+//! MAC table entry for TAP mode.
 typedef struct MACTable
 {
    uint16_t family;
@@ -472,6 +561,7 @@ typedef struct MACTable
    time_t age;
 } MACTable_t;
 
+//! NDP protocol header.
 typedef struct ndp6
 {
    struct ether_header eth;
@@ -485,6 +575,7 @@ typedef struct ndp6
    //struct nd_opt_hdr ndp_opt;
 } __attribute__((packed)) ndp6_t;
 
+//! Structure to hold socket address as a literal string.
 struct sockaddr_str
 {
    sa_family_t sstr_family;
@@ -532,9 +623,6 @@ extern char *tun_dev_;
 #define TUN_DEV "STDIO"
 #endif
 
-extern pthread_mutex_t thread_mutex_;
-extern OcatThread_t *octh_;
-
 /* ocat.c */
 
 
@@ -583,6 +671,11 @@ void set_select_timeout(struct timeval *);
 void set_nonblock(int);
 void set_tunheader(char *, uint32_t);
 uint32_t get_tunheader(char *);
+#ifdef WITH_LOOPBACK_RESPONDER
+void *local_loopback_responder(void *);
+void *remote_loopback_responder(void *);
+int add_remote_loopback_route(void);
+#endif
 
 /* ocatthread.c */
 const OcatThread_t *init_ocat_thread(const char *);
@@ -594,6 +687,8 @@ void detach_thread(void);
 void print_threads(FILE *);
 int term_req(void);
 void set_term_req(void);
+int wait_thread_by_name_ready(const char *);
+int set_thread_ready(void);
 
 /* ocatcompat.c */
 #ifndef HAVE_STRLCAT
@@ -640,13 +735,15 @@ char *ether_ntoa_r(const struct ether_addr *, char *);
 #endif
 uint16_t checksum(const uint16_t *, int);
 void free_ckbuf(uint16_t *);
-uint16_t *malloc_ckbuf(const struct in6_addr *, const struct in6_addr *, uint16_t, uint8_t, const void *);
+uint16_t *malloc_ckbuf(struct in6_addr, struct in6_addr, uint16_t, uint8_t, const void *);
 
 /* ocatsocks.c */
 void socks_queue(struct in6_addr, int);
 void print_socks_queue(FILE *);
 void sig_socks_connector(void);
 void *socks_connector_sel(void *);
+int test_socks_server(void);
+int synchron_socks_connect(const struct in6_addr *);
 
 /* ocatlibe.c */
 void oe_close(int);
@@ -661,6 +758,7 @@ int fdprintf(int, const char *, va_list);
 struct in6_addr *ipv6_lookup_route(const struct in6_addr *);
 void ipv6_print_routes(FILE *);
 int ipv6_parse_route(const char *);
+int ipv6_add_route(const IPv6Route_t *);
 
 #ifdef __CYGWIN__
 /* ocat_wintuntap.c */
